@@ -58,7 +58,62 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
         )
     }
 
+    let output_type = match input.sig.clone().output {
+        syn::ReturnType::Type(_, boxed) => match Box::into_inner(boxed) {
+            syn::Type::Path(syn::TypePath { path: p, .. }) => match p.segments.first() {
+                Some(syn::PathSegment {
+                    arguments:
+                        syn::PathArguments::AngleBracketed(syn::AngleBracketedGenericArguments {
+                            args: a,
+                            ..
+                        }),
+                    ..
+                }) => match a.first() {
+                    Some(a) => match a {
+                        syn::GenericArgument::Type(syn::Type::Path(syn::TypePath {
+                            path: p,
+                            ..
+                        })) => {
+                            if let Some(syn::PathSegment { ident: i, .. }) = p.segments.first() {
+                                Some(i.to_string())
+                            } else {
+                                None
+                            }
+                        }
+                        _ => None,
+                    },
+                    _ => None,
+                },
+                _ => None,
+            },
+            _ => None,
+        },
+        _ => None,
+    };
+
     match attr.to_string().to_lowercase().as_str() {
+        "auto" if Some("EwasmAny".to_string()) == output_type  => quote! {
+            #[cfg(target_arch = "wasm32")]
+            use sewup::bincode;
+            #[cfg(target_arch = "wasm32")]
+            use sewup::ewasm_api::finish_data;
+            #[cfg(all(not(target_arch = "wasm32"), not(test)))]
+            pub fn main() { compile_error!("The function wrapped with ewasm_main need to be compiled with wasm32 target"); }
+            #[cfg(target_arch = "wasm32")]
+            #[no_mangle]
+            pub fn main() {
+                #input
+                match #name() {
+                    Ok(r) =>  {
+                        finish_data(&r.bin);
+                    },
+                    Err(e) => {
+                        let error_msg = e.to_string();
+                        finish_data(&error_msg.as_bytes());
+                    }
+                }
+            }
+        },
         // Return the inner structure from unwrap result
         // This is for a scenario that you take care the result but not using Rust client
         "auto" => quote! {
@@ -76,7 +131,6 @@ pub fn ewasm_main(attr: TokenStream, item: TokenStream) -> TokenStream {
                     Ok(r) =>  {
                         let bin = bincode::serialize(&r).expect("The resuslt of `ewasm_main` should be serializable");
                         finish_data(&bin);
-
                     },
                     Err(e) => {
                         let error_msg = e.to_string();
@@ -161,7 +215,7 @@ pub fn ewasm_fn(_attr: TokenStream, item: TokenStream) -> TokenStream {
         })
         .map(|ty| match ty {
             syn::Type::Path(tp) => (tp.path.segments.first().unwrap().ident.clone(), false),
-            syn::Type::Reference(tr) => match Box::into_inner(tr.elem.clone()) {
+            syn::Type::Reference(tr) => match Box::into_inner(tr.elem) {
                 syn::Type::Path(tp) => (tp.path.segments.first().unwrap().ident.clone(), true),
                 _ => abort_call_site!("please pass Path type or Reference type to ewasm_fn_sig"),
             },
@@ -238,7 +292,7 @@ pub fn ewasm_lib_fn(_attr: TokenStream, item: TokenStream) -> TokenStream {
         })
         .map(|ty| match ty {
             syn::Type::Path(tp) => (tp.path.segments.first().unwrap().ident.clone(), false),
-            syn::Type::Reference(tr) => match Box::into_inner(tr.elem.clone()) {
+            syn::Type::Reference(tr) => match Box::into_inner(tr.elem) {
                 syn::Type::Path(tp) => (tp.path.segments.first().unwrap().ident.clone(), true),
                 _ => abort_call_site!("please pass Path type or Reference type to ewasm_fn_sig"),
             },
@@ -336,12 +390,12 @@ pub fn ewasm_fn_sig(item: TokenStream) -> TokenStream {
                 .split(',')
                 .map(|p| {
                     let p_split = p.split(':').collect::<Vec<_>>();
-                    return if p_split.len() == 2 {
+                    if p_split.len() == 2 {
                         p_split[1]
                     } else {
                         p_split[0]
                     }
-                    .trim();
+                    .trim()
                 })
                 .collect::<Vec<_>>()
                 .join(",")
@@ -550,159 +604,266 @@ pub fn derive_value(item: TokenStream) -> TokenStream {
 #[cfg(feature = "rdb")]
 #[proc_macro_derive(Table)]
 pub fn derive_table(item: TokenStream) -> TokenStream {
-    let re = Regex::new(
-        r"^(pub)\s+struct (?P<name>\w+)(?P<to_first_bracket>[^\{]*\{)(?P<fields>[^\}]*)}",
-    )
-    .unwrap();
-    if let Some(cap) = re.captures(&item.to_string()) {
-        let struct_name = cap.name("name").unwrap().as_str();
-        let field_part = cap.name("fields").unwrap().as_str();
-        let fields = field_part
-            .split(',')
-            .map(|p| p.split(':').nth(0).unwrap_or("").trim())
-            .filter(|s| !s.is_empty())
-            .collect::<Vec<_>>();
-        format!(
-            r#"
-            impl sewup::rdb::traits::Record for {} {{}}
+    let input = syn::parse_macro_input!(item as syn::DeriveInput);
+    let struct_name = &input.ident;
+    let fields_with_type = match &input.data {
+        syn::Data::Struct(syn::DataStruct {
+            fields: syn::Fields::Named(f),
+            ..
+        }) => f
+            .clone()
+            .named
+            .into_pairs()
+            .map(|p| p.into_value())
+            .map(|f| (f.ident.unwrap(), f.ty))
+            .collect::<Vec<_>>(),
+        _ => abort!(&input.ident, "Table derive only use for struct"),
+    };
 
-            #[derive(Default, Clone, sewup::Serialize, sewup::Deserialize)]
-            pub struct {}Wrapper {{
-                id: Option<usize>,
-                {}
-            }}
+    let mut wrapper_fields = vec![(
+        Ident::new("id", Span::call_site()),
+        syn::Type::Path(syn::TypePath {
+            qself: None,
+            path: syn::parse("Option<usize>".parse().unwrap()).unwrap(),
+        }),
+    )];
+    wrapper_fields.append(
+        &mut fields_with_type
+            .iter()
+            .map(|(f, t)| {
+                (
+                    f.clone(),
+                    syn::parse(quote!(Option<#t>).to_string().parse().unwrap()).unwrap(),
+                )
+            })
+            .collect::<Vec<_>>(),
+    );
+    let wrapper_field_names = wrapper_fields.iter().map(|(f, _)| f);
+    let wrapper_field_types = wrapper_fields.iter().map(|(_, t)| t);
+    let field_names = fields_with_type.iter().map(|(f, _)| f);
+    let clone_field_names = field_names.clone();
+    let clone_field_names2 = field_names.clone();
+    let clone_field_names3 = field_names.clone();
+    let clone_field_names4 = field_names.clone();
+    let field_types = fields_with_type.iter().map(|(_, t)| t);
 
-            impl From<{}> for {}Wrapper {{
-                fn from(instance: {}) -> Self {{
-                    Self {{
-                        id: None,
-                        {}
-                    }}
-                }}
-            }}
+    let protocol_name = Ident::new(&format!("{}Protocol", struct_name), Span::call_site());
+    let wrapper_name = Ident::new(&format!("{}Wrapper", struct_name), Span::call_site());
+    let captal_mod_name = Ident::new(
+        &format!("{}", struct_name).to_ascii_uppercase(),
+        Span::call_site(),
+    );
+    let lower_mod_name = Ident::new(
+        &format!("{}", struct_name).to_ascii_lowercase(),
+        Span::call_site(),
+    );
 
-            impl From<{}Wrapper> for {} {{
-                fn from(wrapper: {}Wrapper) -> Self {{
-                    Self {{
-                        {}
-                    }}
-                }}
-            }}
+    quote!(
+        impl sewup::rdb::traits::Record for #struct_name {}
 
-            mod {} {{
-                use sewup_derive::ewasm_fn_sig;
+        #[derive(Clone, sewup::Serialize, sewup::Deserialize)]
+        pub struct #protocol_name {
+            pub select_fields: Option<std::collections::HashSet::<String>>,
+            pub filter: bool,
+            pub records: Vec<#wrapper_name>
+        }
 
-                pub(crate) const GET_SIG: [u8; 4] = ewasm_fn_sig!({}::get());
-                pub(crate) const CREATE_SIG: [u8; 4] = ewasm_fn_sig!({}::create());
-                pub(crate) const UPDATE_SIG: [u8; 4] = ewasm_fn_sig!({}::update());
-                pub(crate) const DELETE_SIG: [u8; 4] = ewasm_fn_sig!({}::delete());
-            }}
+        impl sewup::primitives::IntoEwasmAny for #protocol_name {
+            fn into_ewasm_any(self) -> sewup::primitives::EwasmAny {
+                sewup::primitives::EwasmAny {
+                    bin: sewup::bincode::serialize(&self).expect("The input should be serializable")
+                }
+            }
+        }
 
-            #[cfg(target_arch = "wasm32")]
-            mod {} {{
-                use super::*;
-                pub fn get(instance: {}Wrapper) -> Result<()> {{
-                    let table = sewup::rdb::Db::load(None)?.table::<{}>()?;
-                    let raw_output = table.get_record(instance.id.unwrap_or_default())?;
-                    let mut output: {}Wrapper = raw_output.into();
-                    output.id = instance.id;
-                    sewup::utils::ewasm_return(ewasm_output_from!(output));
-                    Ok(())
-                }}
-                pub fn create(instance: {}Wrapper) -> Result<()> {{
-                    let mut table = sewup::rdb::Db::load(None)?.table::<{}>()?;
-                    let mut output = instance.clone();
-                    output.id = Some(table.add_record(instance.into())?);
-                    table.commit()?;
-                    sewup::utils::ewasm_return(ewasm_output_from!(output));
-                    Ok(())
-                }}
-                pub fn update(instance: {}Wrapper) -> Result<()> {{
-                    let mut table = sewup::rdb::Db::load(None)?.table::<{}>()?;
-                    let id = instance.id.unwrap_or_default();
-                    table.update_record(id, Some(instance.clone().into()))?;
-                    table.commit()?;
-                    sewup::utils::ewasm_return(ewasm_output_from!(instance));
-                    Ok(())
-                }}
-                pub fn delete(instance: {}Wrapper) -> Result<()> {{
-                    let mut table = sewup::rdb::Db::load(None)?.table::<{}>()?;
-                    let id = instance.id.unwrap_or_default();
-                    table.update_record(id, None)?;
-                    table.commit()?;
-                    sewup::utils::ewasm_return(ewasm_output_from!(instance));
-                    Ok(())
-                }}
-            }}
+        impl #protocol_name {
+            pub fn set_select_fields(&mut self, fields: Vec<String>) {
+                if fields.is_empty() {
+                    self.select_fields = None;
+                } else {
+                    let mut select_fields = std::collections::HashSet::<String>::new();
+                    for field in fields.iter() {
+                        select_fields.insert(field.into());
+                    }
+                    self.select_fields = Some(select_fields);
+                }
+            }
+        }
 
-            #[cfg(not(target_arch = "wasm32"))]
-            mod {} {{
-                use super::*;
-                #[inline]
-                pub fn protocol(instance: {}) -> {}Wrapper {{
-                    instance.into()
-                }}
-                pub type Protocol = {}Wrapper;
-                impl Protocol {{
-                    pub fn set_id(&mut self, id: usize) {{
-                        self.id = Some(id);
-                    }}
-                    pub fn is_empty(&self) -> bool {{
-                        self.trusted.is_none() && self.age.is_none()
-                    }}
-                }}
-            }}"#,
-            //Record trait
-            struct_name,
-            // Wraper struct
-            struct_name,
-            field_part.replace(":", ":Option<").replace(",", ">,"),
-            // impl From for warper
-            struct_name,
-            struct_name,
-            struct_name,
-            fields
-                .iter()
-                .map(|f| format!("{}:Some(instance.{})", f, f))
-                .collect::<Vec<_>>()
-                .join(","),
-            // impl From for instance
-            struct_name,
-            struct_name,
-            struct_name,
-            fields
-                .iter()
-                .map(|f| format!(r#"{}:wrapper.{}.expect("{} field missing")"#, f, f, f))
-                .collect::<Vec<_>>()
-                .join(","),
-            // mod for signature
-            struct_name.to_ascii_uppercase(),
-            struct_name,
-            struct_name,
-            struct_name,
-            struct_name,
-            // mod for handlers
-            struct_name.to_ascii_lowercase(),
-            struct_name,
-            struct_name,
-            struct_name,
-            struct_name,
-            struct_name,
-            struct_name,
-            struct_name,
-            struct_name,
-            struct_name,
-            // mod for protocol
-            struct_name.to_ascii_lowercase(),
-            struct_name,
-            struct_name,
-            struct_name,
-        )
-        .parse()
-        .unwrap()
-    } else {
-        abort_call_site!("sewup-derive parsing struct fails, currently we only support flat and simple structure");
-    }
+        impl Default for #protocol_name {
+            fn default() -> Self {
+                Self {
+                    select_fields: None,
+                    filter: false,
+                    records: vec![Default::default()]
+                }
+            }
+        }
+        impl From<#struct_name> for #protocol_name {
+            fn from(instance: #struct_name) -> Self {
+                Self {
+                    select_fields: None,
+                    filter: false,
+                    records: vec![instance.into()]
+                }
+            }
+        }
+
+        impl From<Vec<#struct_name>> for #protocol_name {
+            fn from(instances: Vec<#struct_name>) -> Self {
+                Self {
+                    select_fields: None,
+                    filter: false,
+                    records: instances.into_iter().map(|i| i.into()).collect::<Vec<_>>()
+                }
+            }
+        }
+        impl From<Vec<#wrapper_name>> for #protocol_name {
+            fn from(records: Vec<#wrapper_name>) -> Self {
+                Self {
+                    select_fields: None,
+                    filter: false,
+                    records,
+                }
+            }
+        }
+        pub mod #captal_mod_name {
+            use sewup_derive::ewasm_fn_sig;
+            pub(crate) const GET_SIG: [u8; 4] = ewasm_fn_sig!(#struct_name::get());
+            pub(crate) const CREATE_SIG: [u8; 4] = ewasm_fn_sig!(#struct_name::create());
+            pub(crate) const UPDATE_SIG: [u8; 4] = ewasm_fn_sig!(#struct_name::update());
+            pub(crate) const DELETE_SIG: [u8; 4] = ewasm_fn_sig!(#struct_name::delete());
+        }
+
+        #[derive(Default, Clone, Copy, sewup::Serialize, sewup::Deserialize)]
+        pub struct #wrapper_name {
+            #(pub #wrapper_field_names: #wrapper_field_types,)*
+        }
+        impl From<#struct_name> for #wrapper_name {
+            fn from(instance: #struct_name) -> Self {
+                Self {
+                    id: None,
+                    #(#field_names: Some(instance.#field_names),)*
+                }
+            }
+        }
+        impl From<#wrapper_name> for #struct_name {
+            fn from(wrapper: #wrapper_name) -> Self {
+                Self {
+                    #(#clone_field_names: wrapper.#clone_field_names.expect("#clone_field_names field missing"),)*
+                }
+            }
+        }
+        #[cfg(target_arch = "wasm32")]
+        pub mod #lower_mod_name {
+            use super::*;
+            use sewup::primitives::IntoEwasmAny;
+            pub type Protocol = #protocol_name;
+            pub type Wrapper = #wrapper_name;
+            pub type _Instance = #struct_name;
+            pub fn get(proc: Protocol) -> sewup::Result<sewup::primitives::EwasmAny> {
+                let table = sewup::rdb::Db::load(None)?.table::<_Instance>()?;
+                if proc.filter {
+                    let mut raw_output: Vec<Wrapper> = Vec::new();
+                    for r in table.all_records()?.drain(..){
+                        let mut all_field_match = true;
+                        #(
+                            paste::paste! {
+                                let [<#clone_field_names2 _filed_filter>] : Option<#field_types> =
+                                    sewup::utils::get_field_by_name(proc.records[0], stringify!(#clone_field_names2));
+
+                                let  [<#clone_field_names2 _field>] : #field_types =
+                                    sewup::utils::get_field_by_name(&r, stringify!(#clone_field_names2));
+
+                                if [<#clone_field_names2 _filed_filter>].is_some() {
+                                    all_field_match &=
+                                        [<#clone_field_names2 _filed_filter>].unwrap()
+                                            == [<#clone_field_names2 _field>];
+                                }
+                            }
+                         )*
+
+                        if all_field_match {
+                            let mut wrapper: Wrapper = r.into();
+                            raw_output.push(wrapper);
+                        }
+                    }
+                    if let Some(select_fields) = proc.select_fields {
+                        for w in raw_output.iter_mut() {
+                            #(
+                                if ! select_fields.contains(stringify!(#clone_field_names3)) {
+                                    w.#clone_field_names3 = None;
+                                }
+                             )*
+                        }
+                    }
+                    let p: #protocol_name = raw_output.into();
+                    Ok(p.into_ewasm_any())
+                } else {
+                    let raw_output = table.get_record(proc.records[0].id.unwrap_or_default())?;
+                    let mut output_proc: Protocol = raw_output.into();
+                    output_proc.records[0].id = proc.records[0].id;
+                    Ok(output_proc.into_ewasm_any())
+                }
+            }
+            pub fn create(proc: Protocol) -> sewup::Result<sewup::primitives::EwasmAny> {
+                let mut table = sewup::rdb::Db::load(None)?.table::<_Instance>()?;
+                let mut output_proc = proc.clone();
+                output_proc.records[0].id = Some(table.add_record(proc.records[0].into())?);
+                table.commit()?;
+                Ok(output_proc.into_ewasm_any())
+            }
+            pub fn update(proc: Protocol) -> sewup::Result<sewup::primitives::EwasmAny> {
+                let mut table = sewup::rdb::Db::load(None)?.table::<_Instance>()?;
+                let id = proc.records[0].id.unwrap_or_default();
+                table.update_record(id, Some(proc.records[0].clone().into()))?;
+                table.commit()?;
+                Ok(proc.into_ewasm_any())
+            }
+            pub fn delete(proc: Protocol) -> sewup::Result<sewup::primitives::EwasmAny> {
+                let mut table = sewup::rdb::Db::load(None)?.table::<_Instance>()?;
+                let id = proc.records[0].id.unwrap_or_default();
+                table.update_record(id, None)?;
+                table.commit()?;
+                Ok(proc.into_ewasm_any())
+            }
+        }
+        #[cfg(not(target_arch = "wasm32"))]
+        pub mod #lower_mod_name {
+            use super::*;
+            pub type Protocol = #protocol_name;
+            pub type Wrapper = #wrapper_name;
+            pub type _Instance = #struct_name;
+            pub type Query = Wrapper;
+
+            #[inline]
+            pub fn protocol(instance: _Instance) -> Protocol {
+                instance.into()
+            }
+            impl Protocol {
+                pub fn set_id(&mut self, id: usize) {
+                    self.records[0].id = Some(id);
+                }
+                pub fn is_empty(&self) -> bool {
+                    #(self.records[0].#clone_field_names4.is_none() && )*
+                    true
+                }
+            }
+            pub fn query(instance: _Instance) -> Wrapper {
+                instance.into()
+            }
+
+            impl From<Query> for Protocol {
+                fn from(instance: Query) -> Self {
+                    Self {
+                        select_fields: None,
+                        filter: true,
+                        records: vec![instance.into()]
+                    }
+                }
+            }
+        }
+    ).into()
 }
 
 /// helps you setup the test mododule, and test cases in contract.
@@ -743,7 +904,7 @@ pub fn ewasm_test(attr: TokenStream, item: TokenStream) -> TokenStream {
         let runtime_log_option = if attr_str.is_empty() {
             "".to_string()
         } else {
-            let options = attr_str.split("=").collect::<Vec<_>>();
+            let options = attr_str.split('=').collect::<Vec<_>>();
             match options[0].to_lowercase().as_str() {
                 "log" => format!(".set_log_file({:?}.into())", options[1]),
                 _ => abort_call_site!("no support option"),
